@@ -45,11 +45,18 @@ logger = logging.getLogger(__name__)
 
 # For the time being. This is JAR specific and should probably be configurable.
 ALG_PARAMS = {
-    "sign": ["request_object_signing_alg", "request_object_signing_alg_values_supported", ],
-    "enc_alg": ["request_object_encryption_alg",
-                "request_object_encryption_alg_values_supported", ],
-    "enc_enc": ["request_object_encryption_enc",
-                "request_object_encryption_enc_values_supported", ],
+    "sign": [
+        "request_object_signing_alg",
+        "request_object_signing_alg_values_supported",
+    ],
+    "enc_alg": [
+        "request_object_encryption_alg",
+        "request_object_encryption_alg_values_supported",
+    ],
+    "enc_enc": [
+        "request_object_encryption_enc",
+        "request_object_encryption_enc_values_supported",
+    ],
 }
 
 FORM_POST = """<html>
@@ -83,10 +90,10 @@ def max_age(request):
 
 
 def verify_uri(
-        endpoint_context: EndpointContext,
-        request: Union[dict, Message],
-        uri_type: str,
-        client_id: Optional[str] = None,
+    endpoint_context: EndpointContext,
+    request: Union[dict, Message],
+    uri_type: str,
+    client_id: Optional[str] = None,
 ):
     """
     A redirect URI
@@ -116,20 +123,25 @@ def verify_uri(
         raise URIError("Contains fragment")
 
     (_base, _query) = split_uri(_redirect_uri)
-    # if _query:
-    #     _query = parse_qs(_query)
 
     # Get the clients registered redirect uris
     client_info = endpoint_context.cdb.get(_cid)
     if client_info is None:
         raise KeyError("No such client")
 
-    redirect_uris = client_info.get("{}s".format(uri_type))
+    redirect_uris = client_info.get(f"{uri_type}s")
+
     if redirect_uris is None:
-        raise ValueError(f"No registered {uri_type} for {_cid}")
+        raise RedirectURIError(f"No registered {uri_type} for {_cid}")
     else:
         match = False
-        for regbase, rquery in redirect_uris:
+        for _item in redirect_uris:
+            if isinstance(_item, str):
+                regbase = _item
+                rquery = {}
+            else:
+                regbase, rquery = _item
+
             # The URI MUST exactly match one of the Redirection URI
             if _base == regbase:
                 # every registered query component must exist in the uri
@@ -177,7 +189,7 @@ def join_query(base, query):
 
 
 def get_uri(endpoint_context, request, uri_type):
-    """ verify that the redirect URI is reasonable.
+    """verify that the redirect URI is reasonable.
 
     :param endpoint_context: An EndpointContext instance
     :param request: The Authorization request
@@ -208,7 +220,10 @@ def get_uri(endpoint_context, request, uri_type):
 
 
 def authn_args_gather(
-        request: Union[AuthorizationRequest, dict], authn_class_ref: str, cinfo: dict, **kwargs,
+    request: Union[AuthorizationRequest, dict],
+    authn_class_ref: str,
+    cinfo: dict,
+    **kwargs,
 ):
     """
     Gather information to be used by the authentication method
@@ -251,9 +266,7 @@ def check_unknown_scopes_policy(request_info, client_id, endpoint_context):
         return
 
     scope = request_info["scope"]
-    filtered_scopes = set(
-        endpoint_context.scopes_handler.filter_scopes(scope, client_id=client_id)
-    )
+    filtered_scopes = set(endpoint_context.scopes_handler.filter_scopes(scope, client_id=client_id))
     scopes = set(scope)
     # this prevents that authz would be released for unavailable scopes
     if scopes != filtered_scopes:
@@ -377,10 +390,16 @@ class Authorization(Endpoint):
                 )
                 if _ver_request.jwe_header is not None:
                     self.allowed_request_algorithms(
-                        client_id, endpoint_context, _ver_request.jws_header.get("alg"), "enc_alg",
+                        client_id,
+                        endpoint_context,
+                        _ver_request.jws_header.get("alg"),
+                        "enc_alg",
                     )
                     self.allowed_request_algorithms(
-                        client_id, endpoint_context, _ver_request.jws_header.get("enc"), "enc_enc",
+                        client_id,
+                        endpoint_context,
+                        _ver_request.jws_header.get("enc"),
+                        "enc_enc",
                     )
                 # The protected info overwrites the non-protected
                 for k, v in _ver_request.items():
@@ -404,18 +423,18 @@ class Authorization(Endpoint):
         """
         if not request:
             logger.debug("No AuthzRequest")
-            return self.authentication_error_response(request,
-                                                      error="invalid_request",
-                                                      error_description="Can not parse AuthzRequest"
-                                                      )
+            return self.authentication_error_response(
+                request, error="invalid_request", error_description="Can not parse AuthzRequest"
+            )
 
         request = self.filter_request(endpoint_context, request)
 
         _cinfo = endpoint_context.cdb.get(client_id)
         if not _cinfo:
             logger.error("Client ID ({}) not in client database".format(request["client_id"]))
-            return self.authentication_error_response(request, error="unauthorized_client",
-                                                      error_description="unknown client")
+            return self.authentication_error_response(
+                request, error="unauthorized_client", error_description="unknown client"
+            )
 
         # Is the asked for response_type among those that are permitted
         if not self.verify_response_type(request, _cinfo):
@@ -452,9 +471,7 @@ class Authorization(Endpoint):
             try:
                 res = pick_auth(_context, request)
             except Exception as exc:
-                logger.exception(
-                    f"An error occurred while picking the authN broker: {exc}"
-                )
+                logger.exception(f"An error occurred while picking the authN broker: {exc}")
         if res:
             return res
         else:
@@ -468,7 +485,11 @@ class Authorization(Endpoint):
     def create_session(self, request, user_id, acr, time_stamp, authn_method):
         _context = self.server_get("endpoint_context")
         _mngr = _context.session_manager
-        authn_event = create_authn_event(user_id, authn_info=acr, time_stamp=time_stamp, )
+        authn_event = create_authn_event(
+            user_id,
+            authn_info=acr,
+            time_stamp=time_stamp,
+        )
         _exp_in = authn_method.kwargs.get("expires_in")
         if _exp_in and "valid_until" in authn_event:
             authn_event["valid_until"] = utc_time_sans_frac() + _exp_in
@@ -494,15 +515,26 @@ class Authorization(Endpoint):
         logger.debug("Login required error: {}".format(_res))
         return _res
 
+    def _unwrap_identity(self, identity):
+        if isinstance(identity, dict):
+            try:
+                _id = b64d(as_bytes(identity["uid"]))
+            except BadSyntax:
+                return identity
+        else:
+            _id = b64d(as_bytes(identity))
+
+        return json.loads(as_unicode(_id))
+
     def setup_auth(
-            self,
-            request: Optional[Union[Message, dict]],
-            redirect_uri: str,
-            cinfo: dict,
-            cookie: List[dict] = None,
-            acr: str = None,
-            **kwargs,
-    ):
+        self,
+        request: Optional[Union[Message, dict]],
+        redirect_uri: str,
+        cinfo: dict,
+        cookie: List[dict] = None,
+        acr: str = None,
+        **kwargs,
+    ) -> dict:
         """
 
         :param request: The authorization/authentication request
@@ -527,7 +559,7 @@ class Authorization(Endpoint):
                 _max_age = 0
             else:
                 _max_age = max_age(request)
-            logger.debug(f'Max age: {_max_age}')
+            logger.debug(f"Max age: {_max_age}")
             identity, _ts = authn.authenticated_as(
                 client_id, cookie, authorization=_auth_info, max_age=_max_age
             )
@@ -544,18 +576,16 @@ class Authorization(Endpoint):
             _ts = 0
         else:
             if identity:
-                try:  # If identity['uid'] is in fact a base64 encoded JSON string
-                    _id = b64d(as_bytes(identity["uid"]))
-                except BadSyntax:
-                    pass
-                else:
-                    identity = json.loads(as_unicode(_id))
-
+                identity = self._unwrap_identity(identity)
+                _sid = identity.get("sid")
+                if _sid:
                     try:
-                        _csi = _context.session_manager[identity.get("sid")]
+                        _csi = _context.session_manager[_sid]
                     except Revoked:
+                        logger.debug("Authentication session revoked!!")
                         identity = None
                     else:
+                        logger.debug(f"Session info type: {_csi.__class__.__name__}")
                         if _csi.is_active() is False:
                             identity = None
 
@@ -574,7 +604,7 @@ class Authorization(Endpoint):
             else:
                 return {"function": authn, "args": authn_args}
         else:
-            logger.info("Active authentication")
+            logger.info(f"Active authentication: {identity}")
             if re_authenticate(request, authn):
                 # demand re-authentication
                 return {"function": authn, "args": authn_args}
@@ -626,12 +656,12 @@ class Authorization(Endpoint):
         return ""
 
     def response_mode(
-            self,
-            request: Union[dict, AuthorizationRequest],
-            response_args: Optional[AuthorizationResponse] = None,
-            return_uri: Optional[str] = "",
-            fragment_enc: Optional[bool] = None,
-            **kwargs,
+        self,
+        request: Union[dict, AuthorizationRequest],
+        response_args: Optional[Union[dict, AuthorizationResponse]] = None,
+        return_uri: Optional[str] = "",
+        fragment_enc: Optional[bool] = None,
+        **kwargs,
     ) -> dict:
         resp_mode = request["response_mode"]
         if resp_mode == "form_post":
@@ -639,9 +669,21 @@ class Authorization(Endpoint):
                 _args = response_args.to_dict()
             else:
                 _args = response_args
-            msg = FORM_POST.format(inputs=inputs(_args), action=return_uri, )
+
+            if "error" in _args:
+                if not return_uri:
+                    return_uri = _args["return_uri"]
+                    del _args["return_uri"]
+                if "return_type" in _args:
+                    del _args["return_type"]
+
+            msg = FORM_POST.format(inputs=inputs(_args), action=return_uri)
             kwargs.update(
-                {"response_msg": msg, "content_type": "text/html", "response_placement": "body", }
+                {
+                    "response_msg": msg,
+                    "content_type": "text/html",
+                    "response_placement": "body",
+                }
             )
         elif resp_mode == "fragment":
             if fragment_enc is False:
@@ -662,12 +704,19 @@ class Authorization(Endpoint):
         return kwargs
 
     def error_response(self, response_info, request, error, error_description):
-        resp = self.authentication_error_response(request,
-                                                  error=error,
-                                                  error_description=str(error_description))
+        resp = self.authentication_error_response(
+            request, error=error, error_description=str(error_description)
+        )
         response_info["response_args"] = resp
         return response_info
 
+    def error_by_response_mode(self, response_info, request, error, error_description):
+        response_info = self.error_response(response_info, request, error, error_description)
+        if "return_uri" not in response_info:
+            response_info["return_uri"] = request["redirect_uri"]
+        response_info = self.response_mode(request, **response_info)
+        return response_info
+
     def create_authn_response(self, request: Union[dict, Message], sid: str) -> dict:
         """
         :param request:
@@ -702,7 +751,9 @@ class Authorization(Endpoint):
 
             if "code" in request["response_type"]:
                 _code = self.mint_token(
-                    token_class="authorization_code", grant=grant, session_id=_sinfo["session_id"],
+                    token_class="authorization_code",
+                    grant=grant,
+                    session_id=_sinfo["session_id"],
                 )
                 aresp["code"] = _code.value
                 handled_response_type.append("code")
@@ -711,7 +762,9 @@ class Authorization(Endpoint):
 
             if "token" in rtype:
                 _access_token = self.mint_token(
-                    token_class="access_token", grant=grant, session_id=_sinfo["session_id"],
+                    token_class="access_token",
+                    grant=grant,
+                    session_id=_sinfo["session_id"],
                 )
                 aresp["access_token"] = _access_token.value
                 aresp["token_type"] = "Bearer"
@@ -730,11 +783,15 @@ class Authorization(Endpoint):
                 elif {"id_token", "token"}.issubset(rtype):
                     kwargs = {"access_token": _access_token.value}
 
+                if request["response_type"] == ["id_token"]:
+                    kwargs["as_if"] = "userinfo"
+
                 try:
                     id_token = self.mint_token(
                         token_class="id_token",
                         grant=grant,
                         session_id=_sinfo["session_id"],
+                        scope=request["scope"],
                         **kwargs,
                     )
                     # id_token = _context.idtoken.make(sid, **kwargs)
@@ -754,7 +811,8 @@ class Authorization(Endpoint):
             if not_handled:
                 resp = self.authentication_error_response(
                     request,
-                    error="invalid_request", error_description="unsupported_response_type",
+                    error="invalid_request",
+                    error_description="unsupported_response_type",
                 )
                 return {"response_args": resp, "fragment_enc": fragment_enc}
 
@@ -786,8 +844,9 @@ class Authorization(Endpoint):
         try:
             _mngr.set([user_id, client_id, grant_id], grant)
         except Exception as err:
-            return self.error_response(response_info, request, "server_error",
-                                       "{}".format(err.args))
+            return self.error_response(
+                response_info, request, "server_error", "{}".format(err.args)
+            )
 
         logger.debug("response type: %s" % request["response_type"])
 
@@ -799,8 +858,9 @@ class Authorization(Endpoint):
         try:
             redirect_uri = get_uri(_context, request, "redirect_uri")
         except (RedirectURIError, ParameterError) as err:
-            return self.error_response(response_info, request, "invalid_request",
-                                       "{}".format(err.args))
+            return self.error_response(
+                response_info, request, "invalid_request", "{}".format(err.args)
+            )
         else:
             response_info["return_uri"] = redirect_uri
 
@@ -812,8 +872,9 @@ class Authorization(Endpoint):
             try:
                 response_info = self.response_mode(request, **response_info)
             except InvalidRequest as err:
-                return self.error_response(response_info, request, "invalid_request",
-                                           "{}".format(err.args))
+                return self.error_response(
+                    response_info, request, "invalid_request", "{}".format(err.args)
+                )
 
         _cookie_info = _context.new_cookie(
             name=_context.cookie_handler.name["session"],
@@ -837,40 +898,47 @@ class Authorization(Endpoint):
         try:
             resp_info = self.post_authentication(request, session_id, **kwargs)
         except Exception as err:
-            return self.error_response({}, request, "server_error", err)
+            return self.error_by_response_mode({}, request, "server_error", err)
 
         _context = self.server_get("endpoint_context")
 
+        logger.debug(f"resp_info: {resp_info}")
+
         if "check_session_iframe" in _context.provider_info:
             salt = rndstr()
             try:
                 authn_event = _context.session_manager.get_authentication_event(session_id)
             except KeyError:
-                return self.error_response({}, request, "server_error", "No such session")
+                return self.error_by_response_mode({}, request, "server_error", "No such session")
             else:
                 if authn_event.is_valid() is False:
-                    return self.error_response({}, request, "server_error",
-                                               "Authentication has timed out")
+                    return self.error_by_response_mode(
+                        {}, request, "server_error", "Authentication has timed out"
+                    )
 
             _state = b64e(as_bytes(json.dumps({"authn_time": authn_event["authn_time"]})))
 
             _session_cookie_content = _context.new_cookie(
-                name=_context.cookie_handler.name["session_management"], state=as_unicode(_state),
+                name=_context.cookie_handler.name["session_management"],
+                state=as_unicode(_state),
             )
 
             opbs_value = _session_cookie_content["value"]
 
+            if "return_uri" in resp_info:
+                re_uri = resp_info["return_uri"]
+            else:
+                re_uri = request["redirect_uri"]
+
             logger.debug(
                 "compute_session_state: client_id=%s, origin=%s, opbs=%s, salt=%s",
                 request["client_id"],
-                resp_info["return_uri"],
+                re_uri,
                 opbs_value,
                 salt,
             )
 
-            _session_state = compute_session_state(
-                opbs_value, salt, request["client_id"], resp_info["return_uri"]
-            )
+            _session_state = compute_session_state(opbs_value, salt, request["client_id"], re_uri)
 
             if _session_cookie_content:
                 if "cookie" in resp_info:
@@ -878,7 +946,8 @@ class Authorization(Endpoint):
                 else:
                     resp_info["cookie"] = [_session_cookie_content]
 
-            resp_info["response_args"]["session_state"] = _session_state
+            if "response_args" in resp_info:
+                resp_info["response_args"]["session_state"] = _session_state
 
         # Mix-Up mitigation
         if "response_args" in resp_info:
@@ -891,12 +960,12 @@ class Authorization(Endpoint):
         return kwargs
 
     def process_request(
-            self,
-            request: Optional[Union[Message, dict]] = None,
-            http_info: Optional[dict] = None,
-            **kwargs,
+        self,
+        request: Optional[Union[Message, dict]] = None,
+        http_info: Optional[dict] = None,
+        **kwargs,
     ):
-        """ The AuthorizationRequest endpoint
+        """The AuthorizationRequest endpoint
 
         :param http_info: Information on the HTTP request
         :param request: The authorization request as a Message instance
@@ -930,7 +999,10 @@ class Authorization(Endpoint):
         info = self.setup_auth(request, request["redirect_uri"], cinfo, _my_cookies, **kwargs)
 
         if "error" in info:
-            return info
+            if "response_mode" in request:
+                return self.response_mode(request, info)
+            else:
+                return info
 
         _function = info.get("function")
         if not _function:
